@@ -9,13 +9,13 @@ extern crate std;
 #[cfg(feature = "std")]
 use std::error::Error;
 
+use core::borrow::Borrow;
 use core::cmp::Ordering;
 use core::convert::TryFrom;
 use core::fmt;
 use core::hash::{Hash, Hasher};
 use core::hint::unreachable_unchecked;
 use core::iter::{Product, Sum};
-use core::mem;
 use core::num::FpCategory;
 use core::ops::{
     Add, AddAssign, Deref, DerefMut, Div, DivAssign, Mul, MulAssign, Neg, Rem, RemAssign, Sub,
@@ -915,6 +915,20 @@ impl<T: Float> AsRef<T> for NotNan<T> {
     }
 }
 
+impl Borrow<f32> for NotNan<f32> {
+    #[inline]
+    fn borrow(&self) -> &f32 {
+        &self.0
+    }
+}
+
+impl Borrow<f64> for NotNan<f64> {
+    #[inline]
+    fn borrow(&self) -> &f64 {
+        &self.0
+    }
+}
+
 #[allow(clippy::derive_ord_xor_partial_ord)]
 impl<T: Float> Ord for NotNan<T> {
     fn cmp(&self, other: &NotNan<T>) -> Ordering {
@@ -970,6 +984,30 @@ impl TryFrom<f64> for NotNan<f64> {
     }
 }
 
+macro_rules! impl_from_int_primitive {
+    ($primitive:ty, $inner:ty) => {
+        impl From<$primitive> for NotNan<$inner> {
+            fn from(source: $primitive) -> Self {
+                // the primitives with which this macro will be called cannot hold a value that
+                // f64::from would convert to NaN, so this does not hurt invariants
+                NotNan(<$inner as From<$primitive>>::from(source))
+            }
+        }
+    };
+}
+
+impl_from_int_primitive!(i8, f64);
+impl_from_int_primitive!(i16, f64);
+impl_from_int_primitive!(i32, f64);
+impl_from_int_primitive!(u8, f64);
+impl_from_int_primitive!(u16, f64);
+impl_from_int_primitive!(u32, f64);
+
+impl_from_int_primitive!(i8, f32);
+impl_from_int_primitive!(i16, f32);
+impl_from_int_primitive!(u8, f32);
+impl_from_int_primitive!(u16, f32);
+
 impl From<NotNan<f32>> for NotNan<f64> {
     #[inline]
     fn from(v: NotNan<f32>) -> NotNan<f64> {
@@ -1244,7 +1282,7 @@ fn raw_double_bits<F: Float>(f: &F) -> u64 {
         return CANONICAL_ZERO_BITS;
     }
 
-    let exp_u64 = unsafe { mem::transmute::<i16, u16>(exp) } as u64;
+    let exp_u64 = exp as u16 as u64;
     let sign_u64 = if sign > 0 { 1u64 } else { 0u64 };
     (man & MAN_MASK) | ((exp_u64 << 52) & EXP_MASK) | ((sign_u64 << 63) & SIGN_MASK)
 }
@@ -1519,6 +1557,104 @@ mod impl_serde {
         );
     }
 }
+
+#[cfg(feature = "rkyv")]
+mod impl_rkyv {
+    use super::{NotNan, OrderedFloat};
+    #[cfg(not(feature = "std"))]
+    use num_traits::float::FloatCore as Float;
+    #[cfg(feature = "std")]
+    use num_traits::Float;
+    #[cfg(test)]
+    use rkyv::{archived_root, ser::Serializer};
+    use rkyv::{from_archived, Archive, Deserialize, Fallible, Serialize};
+
+    #[cfg(test)]
+    type DefaultSerializer = rkyv::ser::serializers::CoreSerializer<16, 16>;
+    #[cfg(test)]
+    type DefaultDeserializer = rkyv::Infallible;
+
+    impl<T: Float + Archive> Archive for OrderedFloat<T> {
+        type Archived = OrderedFloat<T>;
+
+        type Resolver = ();
+
+        unsafe fn resolve(&self, _: usize, _: Self::Resolver, out: *mut Self::Archived) {
+            out.write(*self);
+        }
+    }
+
+    impl<T: Float + Serialize<S>, S: Fallible + ?Sized> Serialize<S> for OrderedFloat<T> {
+        fn serialize(&self, _: &mut S) -> Result<Self::Resolver, S::Error> {
+            Ok(())
+        }
+    }
+
+    impl<T: Float + Deserialize<T, D>, D: Fallible + ?Sized> Deserialize<OrderedFloat<T>, D>
+        for OrderedFloat<T>
+    {
+        fn deserialize(&self, _: &mut D) -> Result<OrderedFloat<T>, D::Error> {
+            Ok(from_archived!(*self))
+        }
+    }
+
+    impl<T: Float + Archive> Archive for NotNan<T> {
+        type Archived = NotNan<T>;
+
+        type Resolver = ();
+
+        unsafe fn resolve(&self, _: usize, _: Self::Resolver, out: *mut Self::Archived) {
+            out.write(*self);
+        }
+    }
+
+    impl<T: Float + Serialize<S>, S: Fallible + ?Sized> Serialize<S> for NotNan<T> {
+        fn serialize(&self, _: &mut S) -> Result<Self::Resolver, S::Error> {
+            Ok(())
+        }
+    }
+
+    impl<T: Float + Deserialize<T, D>, D: Fallible + ?Sized> Deserialize<NotNan<T>, D> for NotNan<T> {
+        fn deserialize(&self, _: &mut D) -> Result<NotNan<T>, D::Error> {
+            Ok(from_archived!(*self))
+        }
+    }
+
+    #[test]
+    fn test_ordered_float() {
+        let float = OrderedFloat(1.0f64);
+        let mut serializer = DefaultSerializer::default();
+        serializer
+            .serialize_value(&float)
+            .expect("failed to archive value");
+        let len = serializer.pos();
+        let buffer = serializer.into_serializer().into_inner();
+
+        let archived_value = unsafe { archived_root::<OrderedFloat<f64>>(&buffer[0..len]) };
+        assert_eq!(archived_value, &float);
+        let mut deserializer = DefaultDeserializer::default();
+        let deser_float: OrderedFloat<f64> = archived_value.deserialize(&mut deserializer).unwrap();
+        assert_eq!(deser_float, float);
+    }
+
+    #[test]
+    fn test_not_nan() {
+        let float = NotNan(1.0f64);
+        let mut serializer = DefaultSerializer::default();
+        serializer
+            .serialize_value(&float)
+            .expect("failed to archive value");
+        let len = serializer.pos();
+        let buffer = serializer.into_serializer().into_inner();
+
+        let archived_value = unsafe { archived_root::<NotNan<f64>>(&buffer[0..len]) };
+        assert_eq!(archived_value, &float);
+        let mut deserializer = DefaultDeserializer::default();
+        let deser_float: NotNan<f64> = archived_value.deserialize(&mut deserializer).unwrap();
+        assert_eq!(deser_float, float);
+    }
+}
+
 #[cfg(all(feature = "std", feature = "schemars"))]
 mod impl_schemars {
     extern crate schemars;
